abstract class Result<T> {
  T val;
  Error err;
  bool get isOk => this is Ok;
  bool get isErr => this is Err;
  T get unwrap {
    if (isErr) {
      throw err;
    }
    return val;
  }

  T unwrapOrDefault(T default_value) {
    if (isErr) {
      return default_value;
    }
    return val;
  }

  T unwrapOrElse(T Function(Error e) elseFunc) {
    if (isErr) {
      return elseFunc(err);
    }
    return val;
  }
}

class Ok<T> extends Result<T> {
  @override
  T val;
  Ok(this.val);
}

class Err<T> extends Result<T> {
  @override
  Error err;
  Err(this.err);
}
