import 'package:rust_like_result/rust_like_result.dart';
import 'package:test/test.dart';

void main() {
  Result<String> good;
  Err<String> bad;

  setUp(() {
    good = Ok('Good result');
    bad = Err(ArgumentError('Bad argument AAA'));
  });

  // good:
  test('good.isOk return true', () => expect(good.isOk, isTrue));
  test('good.isErr return false', () => expect(good.isErr, isFalse));
  test('ok is Ok type', () => expect(good, isA<Ok>()));
  test('ok is not Err type', () => expect(good, isNot(Err)));
  test('good val is String', () => expect(good.val, isA<String>()));
  test('good err is an null', () => expect(good.err, null));
  test('good can be unwraped to value',
      () => expect(good.unwrap, equals('Good result')));
  test('good unwrapOrDefault return val',
      () => expect(good.unwrapOrDefault('MyDefault'), 'Good result'));
  test('good unwrapOrElse returns value', 
    ()=>expect(good.unwrapOrElse((e)=>'Hi!'), 'Good result'));

  // bad:
  test('bad.isOk return false', () => expect(bad.isOk, isFalse));
  test('bad.isErr return true', () => expect(bad.isErr, isTrue));
  test('bad is not Ok type', () => expect(bad, isNot(Ok)));
  test('bad is Err type', () => expect(bad, isA<Err>()));
  test('bad throws error while unwrap',
      () => expect(() => bad.unwrap, throwsArgumentError));
  test('bad val is null', () => expect(bad.val, null));
  test('bad err is Error', () => expect(bad.err, isA<Error>()));
  test('bad unwrapOrDefault return default',
    ()=>expect(bad.unwrapOrDefault('Bad value here'), 'Bad value here'));
  test('bad unwrapOrElse returns call result', 
    ()=>expect(bad.unwrapOrElse((e)=>'Other val'), 'Other val'));
}
